// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v7.34.1
// source: artfs.proto

package artfs

import (
	manifest "go.chromium.org/build/siso/toolsupport/artfsutil/proto/manifest"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type AddCasFilesReply struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Total number of files added.  Should be equal to the number of files
	// sent via the stream of FileManifests.
	Total         int64 `protobuf:"varint,1,opt,name=total,proto3" json:"total,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AddCasFilesReply) Reset() {
	*x = AddCasFilesReply{}
	mi := &file_artfs_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AddCasFilesReply) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AddCasFilesReply) ProtoMessage() {}

func (x *AddCasFilesReply) ProtoReflect() protoreflect.Message {
	mi := &file_artfs_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AddCasFilesReply.ProtoReflect.Descriptor instead.
func (*AddCasFilesReply) Descriptor() ([]byte, []int) {
	return file_artfs_proto_rawDescGZIP(), []int{0}
}

func (x *AddCasFilesReply) GetTotal() int64 {
	if x != nil {
		return x.Total
	}
	return 0
}

var File_artfs_proto protoreflect.FileDescriptor

const file_artfs_proto_rawDesc = "" +
	"\n" +
	"\vartfs.proto\x12\x05artfs\x1a\x17manifest/manifest.proto\"(\n" +
	"\x10AddCasFilesReply\x12\x14\n" +
	"\x05total\x18\x01 \x01(\x03R\x05total2K\n" +
	"\x05Artfs\x12B\n" +
	"\vAddCasFiles\x12\x16.manifest.FileManifest\x1a\x17.artfs.AddCasFilesReply\"\x00(\x01B>Z<go.chromium.org/build/siso/toolsupport/artfsutil/proto/artfsb\x06proto3"

var (
	file_artfs_proto_rawDescOnce sync.Once
	file_artfs_proto_rawDescData []byte
)

func file_artfs_proto_rawDescGZIP() []byte {
	file_artfs_proto_rawDescOnce.Do(func() {
		file_artfs_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_artfs_proto_rawDesc), len(file_artfs_proto_rawDesc)))
	})
	return file_artfs_proto_rawDescData
}

var file_artfs_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_artfs_proto_goTypes = []any{
	(*AddCasFilesReply)(nil),      // 0: artfs.AddCasFilesReply
	(*manifest.FileManifest)(nil), // 1: manifest.FileManifest
}
var file_artfs_proto_depIdxs = []int32{
	1, // 0: artfs.Artfs.AddCasFiles:input_type -> manifest.FileManifest
	0, // 1: artfs.Artfs.AddCasFiles:output_type -> artfs.AddCasFilesReply
	1, // [1:2] is the sub-list for method output_type
	0, // [0:1] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_artfs_proto_init() }
func file_artfs_proto_init() {
	if File_artfs_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_artfs_proto_rawDesc), len(file_artfs_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_artfs_proto_goTypes,
		DependencyIndexes: file_artfs_proto_depIdxs,
		MessageInfos:      file_artfs_proto_msgTypes,
	}.Build()
	File_artfs_proto = out.File
	file_artfs_proto_goTypes = nil
	file_artfs_proto_depIdxs = nil
}
